package net.blumia.pcmdroid.ui.screen.addserver

import androidx.compose.foundation.layout.*
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.foundation.verticalScroll
import androidx.compose.material.*
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.AddCircle
import androidx.compose.material.icons.filled.Close
import androidx.compose.material.icons.rounded.AddCircle
import androidx.compose.runtime.*
import androidx.compose.runtime.livedata.observeAsState
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.text.input.KeyboardType
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import net.blumia.pcmdroid.model.Server
import net.blumia.pcmdroid.viewmodel.AddServerViewModel

@Composable
fun StepEnterApiUrl(
    viewModel: AddServerViewModel = AddServerViewModel(),
) {
    Column(
        modifier = Modifier.fillMaxSize(),
        horizontalAlignment = Alignment.CenterHorizontally,
    ) {
        Icon(
            modifier = Modifier.size(50.dp),
            imageVector = Icons.Filled.AddCircle,
            contentDescription = null,
        )
        Text(
            modifier = Modifier
                .fillMaxWidth()
                .padding(vertical = 10.dp),
            text = "Add Server",
            textAlign = TextAlign.Center,
            style = MaterialTheme.typography.h5,
        )
        Text(
            modifier = Modifier
                .fillMaxWidth()
                .padding(bottom = 10.dp),
            text = "Add Private Cloud Music server",
            textAlign = TextAlign.Center,
            style = MaterialTheme.typography.subtitle2,
        )

        val urlStr: String by viewModel.apiUrl.observeAsState("")

        OutlinedTextField(
            modifier = Modifier
                .fillMaxWidth()
                .padding(vertical = 10.dp),
            keyboardOptions = KeyboardOptions(
                keyboardType = KeyboardType.Uri,
            ),
            singleLine = true,
            value = urlStr,
            onValueChange = viewModel::setApiUrl,
            label = { Text("Server API Url") },
        )
    }
}

@Composable
fun StepOtherDetail(
    viewModel: AddServerViewModel = AddServerViewModel(),
) {
    Column(
        modifier = Modifier
            .fillMaxSize()
            .verticalScroll(rememberScrollState()),
        horizontalAlignment = Alignment.CenterHorizontally,
    ) {

        val nameStr: String by viewModel.nameStr.observeAsState("")
        val shortNameStr: String by viewModel.shortNameStr.observeAsState("")
        val baseFolderNameStr: String by viewModel.baseFolderNameStr.observeAsState("")
        val mediaBaseUrlStr: String by viewModel.mediaBaseUrlStr.observeAsState("")
        val preferredFormatsStr: String by viewModel.preferredFormatsStr.observeAsState("")

        OutlinedTextField(
            modifier = Modifier
                .fillMaxWidth()
                .padding(vertical = 10.dp),
            value = shortNameStr,
            onValueChange = viewModel::setShortNameStr,
            label = { Text("Server short name") },
        )

        OutlinedTextField(
            modifier = Modifier
                .fillMaxWidth()
                .padding(vertical = 10.dp),
            value = nameStr,
            onValueChange = viewModel::setNameStr,
            label = { Text("Server name") },
        )

        OutlinedTextField(
            modifier = Modifier
                .fillMaxWidth()
                .padding(vertical = 10.dp),
            value = baseFolderNameStr,
            onValueChange = viewModel::setBaseFolderNameStr,
            label = { Text("Base folder name") },
        )

        OutlinedTextField(
            modifier = Modifier
                .fillMaxWidth()
                .padding(vertical = 10.dp),
            value = mediaBaseUrlStr,
            onValueChange = viewModel::setMediaBaseUrlStr,
            label = { Text("Media base url") },
        )

        OutlinedTextField(
            modifier = Modifier
                .fillMaxWidth()
                .padding(vertical = 10.dp),
            value = preferredFormatsStr,
            onValueChange = viewModel::setPreferredFormatsStr,
            label = { Text("Preferred formats") },
        )
    }
}

@Preview(showBackground = true)
@Composable
fun AddServerScreen(
    onCloseActionTriggered: () -> Unit = {},
    onSubmitServerTriggered: (Server) -> Unit = {},
    viewModel: AddServerViewModel = AddServerViewModel(),
) {
    var stepState by remember { mutableStateOf(0) }
    val finalStep = 1

    fun onNextStepButtonTriggered() {
        if (stepState < 1) {
            stepState++
        }
    }

    fun onPreviousStepButtonTriggered() {
        if (stepState > 0) {
            stepState--
        }
    }

    Scaffold(
        topBar = {
            TopAppBar(
                backgroundColor = Color.Transparent,
                elevation = 0.dp,
                title = {},
                navigationIcon = {
                    IconButton(onClick = onCloseActionTriggered) {
                        Icon(imageVector = Icons.Filled.Close, contentDescription = "Close")
                    }
                },
            )
        }
    ) {
        Column(
            modifier = Modifier
                .fillMaxSize()
                .padding(20.dp),
            horizontalAlignment = Alignment.CenterHorizontally
        ) {
            Column(
                modifier = Modifier
                    .fillMaxWidth()
                    .weight(1f)
            ) {

                when (stepState) {
                    0 -> {
                        StepEnterApiUrl(
                            viewModel,
                        )
                    }
                    1 -> {
                        StepOtherDetail(
                            viewModel = viewModel,
                        )
                    }
                }


            }

            Row(
                modifier = Modifier.fillMaxWidth(),
                horizontalArrangement = Arrangement.SpaceBetween
            ) {
                TextButton(
                    onClick = ::onPreviousStepButtonTriggered,
                    enabled = stepState != 0,
                ) {
                    Text("Back")
                }
                Button(
                    onClick = {
                        if (stepState != finalStep) {
                            viewModel.fetchContentFromApiUrl()
                            onNextStepButtonTriggered()
                        } else {
                            val srv = Server(
                                url = viewModel.apiUrl.value!!,
                                name = viewModel.nameStr.value!!,
                                shortName = viewModel.shortNameStr.value!!,
                                baseFolderName = viewModel.baseFolderNameStr.value!!,
                                mediaBaseUrl = viewModel.mediaBaseUrlStr.value!!,
                                preferredFormats = viewModel.preferredFormatsStr.value!!,
                            )
                            onSubmitServerTriggered(srv)
                        }
                    },
                    shape = RoundedCornerShape(50)
                ) {
                    Text(
                        text = if (stepState != finalStep) "Next" else "Done",
                        modifier = Modifier.padding(horizontal = 5.dp)
                    )
                }
            }
        }
    }
}